
#define PAM_SM_AUTH
#define PAM_SM_ACCOUNT
#define PAM_SM_SESSION
#define PAM_SM_PASSWORD

#define SERVICENAME "pam_make"

#include <security/pam_modules.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <stdio.h>
#include <syslog.h>
#include <stdarg.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>

#define DEBUG(...) if (debug) syslog(LOG_DEBUG,"(pam_make) " __VA_ARGS__);

PAM_EXTERN int 
pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, 
	const char **argv)
{
	return PAM_SUCCESS;
}

PAM_EXTERN int
pam_sm_acct_mgmt(pam_handle_t *pamh, int flags,
	int argc, const char **argv)
{
	return PAM_SUCCESS;
}


PAM_EXTERN int 
pam_sm_setcred(pam_handle_t *pamh, int flags, int argc, 
	const char **argv)
{
	return PAM_SUCCESS;
}

PAM_EXTERN int
pam_sm_open_session(pam_handle_t *pamh, int flags,
	int argc, const char **argv)
{
	return PAM_SUCCESS;
}

PAM_EXTERN int
pam_sm_close_session(pam_handle_t *pamh, int flags,
	int argc, const char **argv)
{
	return PAM_SUCCESS;
}

PAM_EXTERN int pam_sm_chauthtok(pam_handle_t *pamh, int flags,
	int argc, const char **argv)
{
	pid_t pid, retval;
	int status, fd, pip[2], i, debug;
	char buf[65536];
	const char *arggv[] = {"/usr/bin/make", "-C", NULL, NULL};
	static char *envp[] = { NULL };

	debug = 0;
	if ((argc != 1) && (argc != 2)) {
		syslog(LOG_WARNING,"(pam_make) wrong options!\n");
		return PAM_SUCCESS;
	}

	for (; argc-- > 0; ++argv) {
		if (!strcmp(*argv,"debug")) {
			debug = 1;
		} else {
			arggv[2] = *argv;
		}
	}

	DEBUG("called with arg: %s\n", arggv[2])

	if (flags & PAM_PRELIM_CHECK) {
		DEBUG("prelim check - skipped\n");
		return PAM_SUCCESS;
	} else {
		DEBUG("make it, uid: %d, euid: %d\n", getuid(), geteuid());
	}

	if (debug)
		pipe (pip);
	pid = fork();
	if (pid < 0) 
		return PAM_AUTHTOK_ERR;
	if (pid > 0) {
		if (debug) {
			close(pip[1]);
			while ((i = read(pip[0], buf, 65535)) > 0) {
				buf[i] = 0;
				DEBUG("%s\n", buf);
			}
		}
		retval = waitpid(pid, &status, 0);
		if (debug)
			close(pip[0]);
		DEBUG("WIFEXITED: %d\n", WIFEXITED(status));
		DEBUG("WEXITSTATUS: %d\n", WEXITSTATUS(status));
		if (retval < 0)
			return PAM_AUTHTOK_ERR;
		return PAM_SUCCESS;
	}

	DEBUG("forked, uid: %d, euid: %d\n", getuid(), geteuid());
	close(0);
	close(1);
	close(2);
	if (debug) {
		dup(pip[1]);
		dup(pip[1]);
		close(pip[0]);
		close(pip[1]);
	} else {
		/* In the child */
		/* To make sure no one is going to interfere with the child */
		/* I connect std{in,out,err} to /dev/null */
		if ((fd = open("/dev/null", O_RDWR)) < 0) {
			return PAM_AUTHTOK_ERR;
		}
		dup(fd);
		dup(fd);
	}
	dup(fd);
	if (setsid() < 0) {
		return PAM_AUTHTOK_ERR;
	}
	execve(arggv[0], arggv, envp);
	return PAM_AUTHTOK_ERR;
}

#ifdef PAM_STATIC
struct pam_module _pam_make_modstruct = {
	"pam_make",
	pam_sm_authenticate,
	pam_sm_setcred,
	pam_sm_acct_mgmt,
	pam_sm_open_session,
	pam_sm_close_session,
	pam_sm_chauthtok
};
#endif

/* closelog first to make sure it'll _really_ go to syslog */

void writelog(const char *fmt)
{
	closelog();
	openlog(SERVICENAME, LOG_PID | LOG_CONS | LOG_NOWAIT, LOG_AUTH);
	syslog(LOG_ERR, fmt);
	closelog();
}
