/* pam_tcpd module */

/*
 * $Id: pam_tcpd.c,v 1.3 2003/08/12 16:57:51 baggins Exp $
 *
 * Written by Max Liccardo <ravel@tiscalinet.it> 1998/7/21
 *
 * $Log: pam_tcpd.c,v $
 * Revision 1.3  2003/08/12 16:57:51  baggins
 * - *_severity variables are exported in latest libwrap
 *
 * Revision 1.2  2000/05/16 13:37:35  baggins
 * - use closelog() just before openlog(), glibc way of caching fhandles sucks
 * - version 0.72.3
 *
 * Revision 1.1  2000/04/25 13:04:59  baggins
 * - pam_tcpd, TCP wrappers authentication module
 *
 * Revision 1.1  1998/12/24 01:08:57  max
 * new mail address
 *
 * Revision 1.0  1998/07/25 17:13:17  max
 * Initial revision
 *
 * Linux:
 * gcc -fPIC -c pam_tcpd.c
 * ld -x --shared -o pam_tcpd.so pam_tcpd.o -lwrap -lpam -lc
 */

#include <stdio.h>
#include <syslog.h>
#include <stdarg.h>
#include <tcpd.h>




/*
 * here, we make a definition for the externally accessible function
 * in this file (this definition is required for static a module
 * but strongly encouraged generally) it is used to instruct the
 * modules include file to define the function prototypes.
 */

#define PAM_SM_AUTH

#include <security/pam_modules.h>

/* some syslogging */
/* closelog first to make sure it'll _really_ go to syslog */

static void _pam_log(int err, const char *format, ...)
{
    va_list args;

    va_start(args, format);
    closelog();
    openlog("PAM-tcpd", LOG_PID, LOG_AUTHPRIV);
    vsyslog(err, format, args);
    va_end(args);
    closelog();
}


/* argument parsing */

#define PAM_DEBUG_ARG       01

static int _pam_parse(int argc, const char **argv)
{
    int ctrl=0;

    /* step through arguments */
    for (ctrl=0; argc-- > 0; ++argv) {

	/* generic options */

	if (!strcmp(*argv,"debug"))
	    ctrl |= PAM_DEBUG_ARG;
	else {
	    _pam_log(LOG_ERR,"pam_parse: unknown option %s",*argv);
	}
    }

    return ctrl;
}

/* --- authentication management functions (only) --- */

PAM_EXTERN
int pam_sm_authenticate(pam_handle_t *pamh,int flags,int argc
			,const char **argv)
{
    struct request_info s_rqinfo;        
    int ctrl;
    int retval;
    char	*lpszRHost,
    		*lpszRUser,
    		*lpszServiceName;

    ctrl = _pam_parse(argc, argv);
    
    
    retval = pam_get_item(pamh,PAM_SERVICE,(const void **)&lpszServiceName);
    if(retval != PAM_SUCCESS)
    {
        _pam_log(LOG_ERR,"cannot obtain service name: %s",
        		pam_strerror(pamh,retval));
        return(PAM_AUTHINFO_UNAVAIL);
    }		

    retval = pam_get_item(pamh,PAM_RHOST,(const void **)&lpszRHost);
    if(retval != PAM_SUCCESS)
    {
        _pam_log(LOG_ERR,"cannot obtain remote host name: %s",
        		pam_strerror(pamh,retval));
        return(PAM_AUTHINFO_UNAVAIL);
    }
    
    retval = pam_get_item(pamh,PAM_RUSER,(const void **)&lpszRUser);
    if((retval != PAM_SUCCESS) && (ctrl & PAM_DEBUG_ARG))
    {
        _pam_log(LOG_DEBUG,"cannot obtain remote username: %s",
        		pam_strerror(pamh,retval));
    }
    
    if (ctrl & PAM_DEBUG_ARG) {
	_pam_log(LOG_DEBUG, "required authetication for SERVICE %s FROM %s@%s"
		 ,lpszServiceName,lpszRUser,lpszRHost);
    }
    
    request_init(&s_rqinfo,RQ_CLIENT_NAME,lpszRHost,
    			   RQ_DAEMON,lpszServiceName,
    			   RQ_USER,lpszRUser,
    			   0);    
    retval = ((hosts_access(&s_rqinfo) == 0) ? PAM_AUTH_ERR : PAM_SUCCESS);			   
    			   
    return retval;
}

PAM_EXTERN
int pam_sm_setcred(pam_handle_t *pamh,int flags,int argc
		   ,const char **argv)
{
    return PAM_SUCCESS;
}


#ifdef PAM_STATIC

/* static module data */

struct pam_module _pam_tcpd_modstruct = {
    "pam_tcpd",
    pam_sm_authenticate,
    pam_sm_setcred,
    NULL,
    NULL,
    NULL,
    NULL,
};

#endif

/* end of module definition */
